@@ -207,7 +207,7 @@ use File::Glob qw(:case);
 
     'Archive::Tar' => {
         'MAINTAINER'   => 'kane',
-        'DISTRIBUTION' => 'BINGOS/Archive-Tar-1.88.tar.gz',
+        'DISTRIBUTION' => 'BINGOS/Archive-Tar-1.90.tar.gz',
         'FILES'        => q[cpan/Archive-Tar],
         'EXCLUDED'     => ['Makefile.PL'],
         'UPSTREAM'     => 'cpan',
@@ -250,7 +250,6 @@ use File::Glob qw(:case);
                 )
         ],
         'UPSTREAM'   => 'cpan',
-        'CUSTOMIZED' => ['t/open.t'],
     },
 
     'AutoLoader' => {
@@ -263,7 +262,7 @@ use File::Glob qw(:case);
 
     'autouse' => {
         'MAINTAINER'   => 'p5p',
-        'DISTRIBUTION' => 'FLORA/autouse-1.06.tar.gz',
+        'DISTRIBUTION' => 'FLORA/autouse-1.07.tar.gz',
         'FILES'        => q[dist/autouse],
         'EXCLUDED'     => [qr{^t/release-.*\.t}],
         'UPSTREAM'     => 'blead',
@@ -288,7 +287,7 @@ use File::Glob qw(:case);
 
     'B::Debug' => {
         'MAINTAINER'   => 'rurban',
-        'DISTRIBUTION' => 'RURBAN/B-Debug-1.17.tar.gz',
+        'DISTRIBUTION' => 'RURBAN/B-Debug-1.18.tar.gz',
         'FILES'        => q[cpan/B-Debug],
         'EXCLUDED'     => ['t/pod.t'],
         'UPSTREAM'     => 'cpan',
@@ -302,7 +301,7 @@ use File::Glob qw(:case);
 
     'B::Lint' => {
         'MAINTAINER'   => 'jjore',
-        'DISTRIBUTION' => 'FLORA/B-Lint-1.13.tar.gz',
+        'DISTRIBUTION' => 'FLORA/B-Lint-1.14.tar.gz',
         'FILES'        => q[dist/B-Lint],
         'EXCLUDED'     => ['t/test.pl'],
         'UPSTREAM'     => 'blead',
@@ -323,7 +322,7 @@ use File::Glob qw(:case);
 
     'bignum' => {
         'MAINTAINER'   => 'rafl',
-        'DISTRIBUTION' => 'FLORA/bignum-0.29.tar.gz',
+        'DISTRIBUTION' => 'FLORA/bignum-0.30.tar.gz',
         'FILES'        => q[dist/bignum],
         'EXCLUDED'     => [
             qr{^inc/Module/},
@@ -343,12 +342,10 @@ use File::Glob qw(:case);
 
     'CGI' => {
         'MAINTAINER'   => 'lstein',
-        'DISTRIBUTION' => 'MARKSTOS/CGI.pm-3.59.tar.gz',
+        'DISTRIBUTION' => 'MARKSTOS/CGI.pm-3.60.tar.gz',
         'FILES'        => q[cpan/CGI],
         'EXCLUDED'     => [
-            qr{^t/lib/Test},
-            qw( cgi-lib_porting.html
-                cgi_docs.html
+            qw( cgi_docs.html
                 examples/WORLD_WRITABLE/18.157.1.253.sav
                 t/gen-tests/gen-start-end-tags.pl
                 t/fast.t
@@ -365,7 +362,7 @@ use File::Glob qw(:case);
 
     'Compress::Raw::Bzip2' => {
         'MAINTAINER'   => 'pmqs',
-        'DISTRIBUTION' => 'PMQS/Compress-Raw-Bzip2-2.052.tar.gz',
+        'DISTRIBUTION' => 'PMQS/Compress-Raw-Bzip2-2.055.tar.gz',
         'FILES'        => q[cpan/Compress-Raw-Bzip2],
         'EXCLUDED'     => [
             qr{^t/Test/},
@@ -376,7 +373,7 @@ use File::Glob qw(:case);
 
     'Compress::Raw::Zlib' => {
         'MAINTAINER'   => 'pmqs',
-        'DISTRIBUTION' => 'PMQS/Compress-Raw-Zlib-2.054.tar.gz',
+        'DISTRIBUTION' => 'PMQS/Compress-Raw-Zlib-2.056.tar.gz',
 
         'FILES'    => q[cpan/Compress-Raw-Zlib],
         'EXCLUDED' => [
@@ -413,6 +410,7 @@ use File::Glob qw(:case);
             qr{^t/CPAN/authors/},
             qw( lib/CPAN/Admin.pm
                 Makefile.PL
+                scripts/cpan-mirrors
                 SlayMakefile
                 t/00signature.t
                 t/04clean_load.t
@@ -424,6 +422,7 @@ use File::Glob qw(:case);
                 t/31sessions.t
                 t/41distribution.t
                 t/42distroprefs.t
+                t/44cpanmeta.t
                 t/43distroprefspref.t
                 t/50pod.t
                 t/51pod.t
@@ -436,8 +435,12 @@ use File::Glob qw(:case);
                 t/CPAN/TestPatch.txt
                 t/CPAN/modules/02packages.details.txt
                 t/CPAN/modules/03modlist.data
+                t/data/META-dynamic.json
                 t/data/META-dynamic.yml
+                t/data/META-static.json
                 t/data/META-static.yml
+                t/data/MYMETA.json
+                t/data/MYMETA.yml
                 t/local_utils.pm
                 t/perlcriticrc
                 t/yaml_code.yml
@@ -448,7 +451,7 @@ use File::Glob qw(:case);
 
     'CPANPLUS' => {
         'MAINTAINER'   => 'kane',
-        'DISTRIBUTION' => 'BINGOS/CPANPLUS-0.9130.tar.gz',
+        'DISTRIBUTION' => 'BINGOS/CPANPLUS-0.9131.tar.gz',
         'FILES'        => q[cpan/CPANPLUS],
         'EXCLUDED'     => [
             qr{^inc/},
@@ -526,7 +529,7 @@ use File::Glob qw(:case);
 
     'DB_File' => {
         'MAINTAINER'   => 'pmqs',
-        'DISTRIBUTION' => 'PMQS/DB_File-1.826.tar.gz',
+        'DISTRIBUTION' => 'PMQS/DB_File-1.827.tar.gz',
         'FILES'        => q[cpan/DB_File],
         'EXCLUDED'     => [
             qr{^patches/},
@@ -609,7 +612,7 @@ use File::Glob qw(:case);
 
     'Dumpvalue' => {
         'MAINTAINER'   => 'p5p',
-        'DISTRIBUTION' => 'FLORA/Dumpvalue-1.16.tar.gz',
+        'DISTRIBUTION' => 'FLORA/Dumpvalue-1.17.tar.gz',
         'FILES'        => q[dist/Dumpvalue],
         'EXCLUDED'     => [qr{^t/release-.*\.t}],
         'UPSTREAM'     => 'blead',
@@ -623,7 +626,7 @@ use File::Glob qw(:case);
 
     'Encode' => {
         'MAINTAINER'   => 'dankogai',
-        'DISTRIBUTION' => 'DANKOGAI/Encode-2.44.tar.gz',
+        'DISTRIBUTION' => 'DANKOGAI/Encode-2.47.tar.gz',
         'FILES'        => q[cpan/Encode],
         'UPSTREAM'     => 'cpan',
     },
@@ -747,7 +750,7 @@ use File::Glob qw(:case);
 
     'ExtUtils::Manifest' => {
         'MAINTAINER'   => 'p5p',
-        'DISTRIBUTION' => 'FLORA/ExtUtils-Manifest-1.60.tar.gz',
+        'DISTRIBUTION' => 'FLORA/ExtUtils-Manifest-1.61.tar.gz',
         'FILES'        => q[dist/ExtUtils-Manifest],
         'EXCLUDED'     => [qr(t/release-.*\.t)],
         'UPSTREAM'     => 'blead',
@@ -762,7 +765,7 @@ use File::Glob qw(:case);
 
     'perlfaq' => {
         'MAINTAINER'   => 'perlfaq',
-        'DISTRIBUTION' => 'LLAP/perlfaq-5.0150040.tar.gz',
+        'DISTRIBUTION' => 'LLAP/perlfaq-5.0150041.tar.gz',
         'FILES'        => q[cpan/perlfaq],
         'EXCLUDED'     => [
             qw( t/release-pod-syntax.t
@@ -944,7 +947,6 @@ use File::Glob qw(:case);
                 t/gol-compat.t
                 ),
         ],
-        'MAP'      => { '' => 'cpan/Getopt-Long/' },
         'UPSTREAM' => 'cpan',
     },
 
@@ -1022,7 +1024,7 @@ use File::Glob qw(:case);
 
     'IO-Compress' => {
         'MAINTAINER'   => 'pmqs',
-        'DISTRIBUTION' => 'PMQS/IO-Compress-2.052.tar.gz',
+        'DISTRIBUTION' => 'PMQS/IO-Compress-2.055.tar.gz',
         'FILES'        => q[cpan/IO-Compress],
         'EXCLUDED'     => [qr{t/Test/}],
         'UPSTREAM'     => 'cpan',
@@ -1098,7 +1100,7 @@ use File::Glob qw(:case);
 
     'Locale-Codes' => {
         'MAINTAINER'   => 'sbeck',
-        'DISTRIBUTION' => 'SBECK/Locale-Codes-3.22.tar.gz',
+        'DISTRIBUTION' => 'SBECK/Locale-Codes-3.23.tar.gz',
         'FILES'        => q[cpan/Locale-Codes],
         'EXCLUDED'     => [
             qw( t/pod_coverage.t
@@ -1244,14 +1246,13 @@ use File::Glob qw(:case);
 
     'Module::Build' => {
         'MAINTAINER'   => 'kwilliams',
-        'DISTRIBUTION' => 'LEONT/Module-Build-0.40.tar.gz',
+        'DISTRIBUTION' => 'LEONT/Module-Build-0.4003.tar.gz',
         'FILES'        => q[cpan/Module-Build],
         'EXCLUDED'     => [
             qw( t/par.t
                 t/signature.t
                 ),
             qr{^contrib/},
-            qr{^devtools},
             qr{^inc},
         ],
         'CUSTOMIZED' => ['lib/Module/Build/ConfigData.pm'],
@@ -1260,7 +1261,7 @@ use File::Glob qw(:case);
 
     'Module::CoreList' => {
         'MAINTAINER'   => 'bingos',
-        'DISTRIBUTION' => 'BINGOS/Module-CoreList-2.68.tar.gz',
+        'DISTRIBUTION' => 'BINGOS/Module-CoreList-2.72.tar.gz',
         'FILES'        => q[dist/Module-CoreList],
         'UPSTREAM'     => 'blead',
     },
@@ -1274,7 +1275,7 @@ use File::Glob qw(:case);
 
     'Module::Load::Conditional' => {
         'MAINTAINER'   => 'kane',
-        'DISTRIBUTION' => 'BINGOS/Module-Load-Conditional-0.50.tar.gz',
+        'DISTRIBUTION' => 'BINGOS/Module-Load-Conditional-0.54.tar.gz',
         'FILES'        => q[cpan/Module-Load-Conditional],
         'UPSTREAM'     => 'cpan',
     },
@@ -1288,7 +1289,7 @@ use File::Glob qw(:case);
 
     'Module::Metadata' => {
         'MAINTAINER'   => 'dagolden',
-        'DISTRIBUTION' => 'DAGOLDEN/Module-Metadata-1.000009.tar.gz',
+        'DISTRIBUTION' => 'APEIRON/Module-Metadata-1.000011.tar.gz',
         'FILES'        => q[cpan/Module-Metadata],
         'EXCLUDED'     => [
             qr{^maint},
@@ -1299,7 +1300,7 @@ use File::Glob qw(:case);
 
     'Module::Pluggable' => {
         'MAINTAINER'   => 'simonw',
-        'DISTRIBUTION' => 'SIMONW/Module-Pluggable-4.0.tar.gz',
+        'DISTRIBUTION' => 'SIMONW/Module-Pluggable-4.3.tar.gz',
         'FILES'        => q[cpan/Module-Pluggable],
         'UPSTREAM'     => 'cpan',
         'CUSTOMIZED'   => ['Makefile.PL'],
@@ -1383,7 +1384,6 @@ use File::Glob qw(:case);
         'MAINTAINER'   => 'dagolden',
         'DISTRIBUTION' => 'DAGOLDEN/Parse-CPAN-Meta-1.4404.tar.gz',
         'FILES'        => q[cpan/Parse-CPAN-Meta],
-        'EXCLUDED'     => [],
         'UPSTREAM'     => 'cpan',
     },
 
@@ -1543,7 +1543,7 @@ use File::Glob qw(:case);
 
     'Pod::Simple' => {
         'MAINTAINER'   => 'arandal',
-        'DISTRIBUTION' => 'DWHEELER/Pod-Simple-3.20.tar.gz',
+        'DISTRIBUTION' => 'DWHEELER/Pod-Simple-3.23.tar.gz',
         'FILES'        => q[cpan/Pod-Simple],
         'UPSTREAM'     => 'cpan',
     },
@@ -1560,7 +1560,6 @@ use File::Glob qw(:case);
         'CUSTOMIZED' => [
             qw( scripts/pod2man.PL
                 scripts/pod2text.PL
-                pod/perlpodstyle.pod
                 ),
         ],
         'MAP' => {
@@ -1637,14 +1636,14 @@ use File::Glob qw(:case);
 
     'Socket' => {
         'MAINTAINER'   => 'pevans',
-        'DISTRIBUTION' => 'PEVANS/Socket-2.002.tar.gz',
+        'DISTRIBUTION' => 'PEVANS/Socket-2.006.tar.gz',
         'FILES'        => q[cpan/Socket],
         'UPSTREAM'     => 'cpan',
     },
 
     'Storable' => {
         'MAINTAINER'   => 'ams',
-        'DISTRIBUTION' => 'AMS/Storable-2.35.tar.gz',
+        'DISTRIBUTION' => 'AMS/Storable-2.39.tar.gz',
         'FILES'        => q[dist/Storable],
         'EXCLUDED'     => [qr{^t/Test/}],
         'UPSTREAM'     => 'blead',
@@ -1658,7 +1657,7 @@ use File::Glob qw(:case);
 
     'Sys::Syslog' => {
         'MAINTAINER'   => 'saper',
-        'DISTRIBUTION' => 'SAPER/Sys-Syslog-0.29.tar.gz',
+        'DISTRIBUTION' => 'SAPER/Sys-Syslog-0.32.tar.gz',
         'FILES'        => q[cpan/Sys-Syslog],
         'EXCLUDED'     => [
             qr{^eg/},
@@ -1669,11 +1668,9 @@ use File::Glob qw(:case);
                 t/podspell.t
                 t/portfs.t
                 win32/PerlLog.RES
-                win32/PerlLog_RES.uu
                 ),
         ],
         'UPSTREAM'   => 'cpan',
-        'CUSTOMIZED' => ['t/syslog.t'],
     },
 
     'Term::ANSIColor' => {
@@ -1706,7 +1703,7 @@ use File::Glob qw(:case);
 
     'Term::ReadLine' => {
         'MAINTAINER'   => 'p5p',
-        'DISTRIBUTION' => 'FLORA/Term-ReadLine-1.09.tar.gz',
+        'DISTRIBUTION' => 'FLORA/Term-ReadLine-1.10.tar.gz',
         'FILES'        => q[dist/Term-ReadLine],
         'EXCLUDED'     => [qr{^t/release-.*\.t}],
         'UPSTREAM'     => 'blead',
@@ -1742,10 +1739,61 @@ use File::Glob qw(:case);
                 t/lib/if.pm
                 ),
         ],
+        'MAP'        => {
+            ''                    => 'cpan/Test-Harness/',
+            't/compat/env.opts.t' => 'cpan/Test-Harness/t/compat/env_opts.t'
+        },
         'UPSTREAM'   => 'cpan',
         'CUSTOMIZED' => [
-            qw( t/source.t
-                t/testargs.t
+            qw( lib/App/Prove.pm
+                lib/App/Prove/State.pm
+                lib/App/Prove/State/Result.pm
+                lib/App/Prove/State/Result/Test.pm
+                lib/TAP/Base.pm
+                lib/TAP/Formatter/Base.pm
+                lib/TAP/Formatter/Color.pm
+                lib/TAP/Formatter/Console.pm
+                lib/TAP/Formatter/Console/ParallelSession.pm
+                lib/TAP/Formatter/Console/Session.pm
+                lib/TAP/Formatter/File.pm
+                lib/TAP/Formatter/File/Session.pm
+                lib/TAP/Formatter/Session.pm
+                lib/TAP/Harness.pm
+                lib/TAP/Object.pm
+                lib/TAP/Parser.pm
+                lib/TAP/Parser/Aggregator.pm
+                lib/TAP/Parser/Grammar.pm
+                lib/TAP/Parser/Iterator.pm
+                lib/TAP/Parser/Iterator/Array.pm
+                lib/TAP/Parser/Iterator/Process.pm
+                lib/TAP/Parser/Iterator/Stream.pm
+                lib/TAP/Parser/IteratorFactory.pm
+                lib/TAP/Parser/Multiplexer.pm
+                lib/TAP/Parser/Result.pm
+                lib/TAP/Parser/Result/Bailout.pm
+                lib/TAP/Parser/Result/Comment.pm
+                lib/TAP/Parser/Result/Plan.pm
+                lib/TAP/Parser/Result/Pragma.pm
+                lib/TAP/Parser/Result/Test.pm
+                lib/TAP/Parser/Result/Unknown.pm
+                lib/TAP/Parser/Result/Version.pm
+                lib/TAP/Parser/Result/YAML.pm
+                lib/TAP/Parser/ResultFactory.pm
+                lib/TAP/Parser/Scheduler.pm
+                lib/TAP/Parser/Scheduler/Job.pm
+                lib/TAP/Parser/Scheduler/Spinner.pm
+                lib/TAP/Parser/Source.pm
+                lib/TAP/Parser/SourceHandler.pm
+                lib/TAP/Parser/SourceHandler/Executable.pm
+                lib/TAP/Parser/SourceHandler/File.pm
+                lib/TAP/Parser/SourceHandler/Handle.pm
+                lib/TAP/Parser/SourceHandler/Perl.pm
+                lib/TAP/Parser/SourceHandler/RawTAP.pm
+                lib/TAP/Parser/Utils.pm
+                lib/TAP/Parser/YAMLish/Reader.pm
+                lib/TAP/Parser/YAMLish/Writer.pm
+                lib/Test/Harness.pm
+                t/source.t
                 ),
         ],
     },
@@ -1769,7 +1817,7 @@ use File::Glob qw(:case);
 
     'Text::Abbrev' => {
         'MAINTAINER'   => 'p5p',
-        'DISTRIBUTION' => 'FLORA/Text-Abbrev-1.01.tar.gz',
+        'DISTRIBUTION' => 'FLORA/Text-Abbrev-1.02.tar.gz',
         'FILES'        => q[dist/Text-Abbrev],
         'EXCLUDED'     => [qr{^t/release-.*\.t}],
         'UPSTREAM'     => 'blead',
@@ -1785,6 +1833,18 @@ use File::Glob qw(:case);
                 t/99_pmv.t
                 ),
         ],
+        'CUSTOMIZED'   => [
+            qw( t/01_compile.t
+                t/02_extbrk.t
+                t/03_extcbk.t
+                t/04_extdel.t
+                t/05_extmul.t
+                t/06_extqlk.t
+                t/07_exttag.t
+                t/08_extvar.t
+                t/09_gentag.t
+                ),
+        ],
         'UPSTREAM' => 'cpan',
     },
 
@@ -1819,15 +1879,18 @@ use File::Glob qw(:case);
 
     'Text-Tabs+Wrap' => {
         'MAINTAINER'   => 'muir',
-        'DISTRIBUTION' => 'MUIR/modules/Text-Tabs+Wrap-2009.0305.tar.gz',
+        'DISTRIBUTION' => 'MUIR/modules/Text-Tabs+Wrap-2012.0818.tar.gz',
         'FILES'        => q[cpan/Text-Tabs],
-        'EXCLUDED'   => ['t/dnsparks.t'],    # see af6492bf9e
-        'UPSTREAM'   => 'cpan',
-        'CUSTOMIZED' => [
-            qw( t/fill.t
-                t/tabs.t
-                ),
+        'EXCLUDED'   => [
+            qr/^lib\.old/,
+            't/dnsparks.t',    # see af6492bf9e
         ],
+        'MAP'          => {
+            ''                        => 'cpan/Text-Tabs/',
+            'lib.modern/Text/Tabs.pm' => 'cpan/Text-Tabs/lib/Text/Tabs.pm',
+            'lib.modern/Text/Wrap.pm' => 'cpan/Text-Tabs/lib/Text/Wrap.pm',
+        },
+        'UPSTREAM'   => 'cpan',
     },
 
     'Thread::Queue' => {
@@ -1874,7 +1937,7 @@ use File::Glob qw(:case);
 
     'threads::shared' => {
         'MAINTAINER'   => 'jdhedden',
-        'DISTRIBUTION' => 'JDHEDDEN/threads-shared-1.40.tar.gz',
+        'DISTRIBUTION' => 'JDHEDDEN/threads-shared-1.41.tar.gz',
         'FILES'        => q[dist/threads-shared],
         'EXCLUDED'     => [
             qw( examples/class.pl
@@ -1927,12 +1990,13 @@ use File::Glob qw(:case);
 
     'Time::Local' => {
         'MAINTAINER'   => 'drolsky',
-        'DISTRIBUTION' => 'FLORA/Time-Local-1.2000.tar.gz',
+        'DISTRIBUTION' => 'DROLSKY/Time-Local-1.2300.tar.gz',
         'FILES'        => q[cpan/Time-Local],
         'EXCLUDED'     => [
             qw( t/pod-coverage.t
                 t/pod.t
                 ),
+            qr{^t/release-.*\.t},
         ],
         'UPSTREAM' => 'cpan',
     },
@@ -1982,7 +2046,7 @@ use File::Glob qw(:case);
 
     'version' => {
         'MAINTAINER'   => 'jpeacock',
-        'DISTRIBUTION' => 'JPEACOCK/version-0.99.tar.gz',
+        'DISTRIBUTION' => 'JPEACOCK/version-0.9901.tar.gz',
         'FILES'        => q[lib/version.pm lib/version.pod lib/version],
         'EXCLUDED' => [
             qr{^t/.*\.t$},
@@ -2034,7 +2098,7 @@ use File::Glob qw(:case);
 
     'Win32' => {
         'MAINTAINER'   => 'jand',
-        'DISTRIBUTION' => "JDB/Win32-0.44.tar.gz",
+        'DISTRIBUTION' => "JDB/Win32-0.45.tar.gz",
         'FILES'        => q[cpan/Win32],
         'UPSTREAM'     => 'cpan',
     },
@@ -2058,7 +2122,7 @@ use File::Glob qw(:case);
 
     'XSLoader' => {
         'MAINTAINER'   => 'saper',
-        'DISTRIBUTION' => 'SAPER/XSLoader-0.15.tar.gz',
+        'DISTRIBUTION' => 'SAPER/XSLoader-0.16.tar.gz',
         'FILES'        => q[dist/XSLoader],
         'EXCLUDED'     => [
             qr{^eg/},
@@ -2068,7 +2132,8 @@ use File::Glob qw(:case);
                 ),
             'XSLoader.pm',    # we use XSLoader_pm.PL
         ],
-        'UPSTREAM' => 'blead',
+        # Revert UPSTREAM to 'blead' after 0.17 is released
+        'UPSTREAM' => undef,
     },
 
     's2p' => {
