@@ -8,7 +8,7 @@ package Win32;
     require DynaLoader;
 
     @ISA = qw|Exporter DynaLoader|;
-    $VERSION = '0.44';
+    $VERSION = '0.45';
     $XS_VERSION = $VERSION;
     $VERSION = eval $VERSION;
 
@@ -686,6 +686,11 @@ Unloads a previously loaded dynamic-link library.  The HANDLE is
 no longer valid after this call.  See L<LoadLibrary|Win32::LoadLibrary(LIBNAME)>
 for information on dynamically loading a library.
 
+=item Win32::GetACP()
+
+Returns the current Windows ANSI code page identifier for the operating
+system.  See also GetOEMCP(), GetConsoleCP() and GetConsoleOutputCP().
+
 =item Win32::GetANSIPathName(FILENAME)
 
 Returns an ANSI version of FILENAME.  This may be the short name
@@ -712,6 +717,20 @@ for the x64 processor and 2200 for the Itanium.  Since it returns the
 native processor type it will return a 64-bit processor type even when
 called from a 32-bit Perl running on 64-bit Windows.
 
+=item Win32::GetConsoleCP()
+
+Returns the input code page used by the console associated with the
+calling process.  To set the console's input code page, see
+SetConsoleCP().  See also GetConsoleOutputCP(), GetACP() and
+GetOEMCP().
+
+=item Win32::GetConsoleOutputCP()
+
+Returns the output code page used by the console associated with the
+calling process.  To set the console's output code page, see
+SetConsoleOutputCP().  See also GetConsoleCP(), GetACP(), and
+GetOEMCP().
+
 =item Win32::GetCwd()
 
 [CORE] Returns the current active drive and directory.  This function
@@ -848,6 +867,12 @@ before passing the path to a system call or another program.
 [CORE] Returns a string in the form of "<d>:" where <d> is the first
 available drive letter.
 
+=item Win32::GetOEMCP()
+
+Returns the current original equipment manufacturer (OEM) code page
+identifier for the operating system.  See also GetACP(), GetConsoleCP()
+and GetConsoleOutputCP().
+
 =item Win32::GetOSDisplayName()
 
 Returns the "marketing" name of the Windows operating system version
@@ -1167,6 +1192,20 @@ The following symbolic constants for SHOWWINDOW are available
 (but not exported) from the Win32 module: SW_HIDE, SW_SHOWNORMAL,
 SW_SHOWMINIMIZED, SW_SHOWMAXIMIZED and SW_SHOWNOACTIVATE.
 
+=item Win32::SetConsoleCP(ID)
+
+Sets the input code page used by the console associated with the
+calling process.  The return value of SetConsoleCP() is nonzero on
+success or zero on failure.  To get the console's input code page, see
+GetConsoleCP().
+
+=item Win32::SetConsoleOutputCP(ID)
+
+Sets the output code page used by the console associated with the
+calling process.  The return value of SetConsoleOutputCP() is nonzero on
+success or zero on failure.  To get the console's output code page, see
+GetConsoleOutputCP().
+
 =item Win32::SetCwd(NEWDIRECTORY)
 
 [CORE] Sets the current active drive and directory.  This function does not
