@@ -33,8 +33,8 @@ apirevision=''
 apisubversion=''
 apiversion=''
 ar='arm-epoc-pe-ar'
-archlib='/usr/lib/perl/5.17.1/epoc'
-archlibexp='/usr/lib/perl/5.17.1/epoc'
+archlib='/usr/lib/perl/5.17.4/epoc'
+archlibexp='/usr/lib/perl/5.17.4/epoc'
 archname64=''
 archname='epoc'
 archobjs='epoc.o epocish.o epoc_stubs.o'
@@ -279,7 +279,10 @@ d_inetntop='undef'
 d_inetpton='undef'
 d_int64_t='undef'
 d_iovec_s='undef'
+d_ip_mreq='undef'
+d_ip_mreq_source='undef'
 d_ipv6_mreq='undef'
+d_ipv6_mreq_source='undef'
 d_isascii='define'
 d_isblank='undef'
 d_isfinite='undef'
@@ -714,16 +717,16 @@ inc_version_list=' '
 inc_version_list_init='0'
 incpath=''
 inews=''
-installarchlib='/home/of/PERL/perl/lib/5.17.1/epoc'
+installarchlib='/home/of/PERL/perl/lib/5.17.4/epoc'
 installbin='/home/of/PERL/System/Programs/'
 installman1dir='/home/of/PERL/man1'
 installman3dir='/home/of/PERL/man3'
 installprefix=''
 installprefixexp=''
-installprivlib='/home/of/PERL/perl/lib/5.17.1/'
+installprivlib='/home/of/PERL/perl/lib/5.17.4/'
 installscript='/home/of/PERL/bin/'
-installsitearch='/home/of/PERL/site/lib/site_perl/5.17.1/epoc'
-installsitelib='/home/of/PERL/perl/lib/site_perl/5.17.1'
+installsitearch='/home/of/PERL/site/lib/site_perl/5.17.4/epoc'
+installsitelib='/home/of/PERL/perl/lib/site_perl/5.17.4'
 installstyle=''
 installusrbinperl='undef'
 installvendorarch=''
@@ -845,8 +848,8 @@ pmake=''
 pr=''
 prefix=''
 prefixexp=''
-privlib='/usr/lib/perl/5.17.1'
-privlibexp='/usr/lib/perl/5.17.1'
+privlib='/usr/lib/perl/5.17.4'
+privlibexp='/usr/lib/perl/5.17.4'
 procselfexe=''
 prototype='define'
 ptrsize='4'
@@ -909,11 +912,11 @@ sig_num='0'
 sig_num_init='0, 0'
 sig_size='1'
 signal_t='void'
-sitearch='/usr/lib/perl/site_perl/5.17.1/epoc'
-sitearchexp='/usr/lib/perl/site_perl/5.17.1/epoc'
-sitelib='/usr/lib/perl/site_perl/5.17.1/'
+sitearch='/usr/lib/perl/site_perl/5.17.4/epoc'
+sitearchexp='/usr/lib/perl/site_perl/5.17.4/epoc'
+sitelib='/usr/lib/perl/site_perl/5.17.4/'
 sitelib_stem='/usr/lib/perl/site_perl'
-sitelibexp='/usr/lib/perl/site_perl/5.17.1/'
+sitelibexp='/usr/lib/perl/site_perl/5.17.4/'
 siteprefix=''
 siteprefixexp=''
 sizesize='4'
@@ -1023,7 +1026,7 @@ vendorlib_stem=''
 vendorlibexp=''
 vendorprefix=''
 vendorprefixexp=''
-version='5.17.1'
+version='5.17.4'
 versiononly='undef'
 vi=''
 voidflags='15'
@@ -1047,10 +1050,10 @@ config_arg10=''
 config_arg11=''
 PERL_REVISION=5
 PERL_VERSION=17
-PERL_SUBVERSION=1
+PERL_SUBVERSION=4
 PERL_API_REVISION=5
 PERL_API_VERSION=17
-PERL_API_SUBVERSION=1
+PERL_API_SUBVERSION=4
 PERL_CONFIG_SH=true
 CONFIGDOTSH=true
 # Variables propagated from previous config.sh file.
