@@ -221,6 +221,11 @@ Deprecated.  Use C<GIMME_V> instead.
   /* OP_RV2[AGH]V, OP_PAD[AH]V, OP_[AH]ELEM, OP_[AH]SLICE OP_AV2ARYLEN,
      OP_R?KEYS, OP_SUBSTR, OP_POS, OP_VEC */
 #define OPpMAYBE_LVSUB		8	/* We might be an lvalue to return */
+  /* OP_RV2HV and OP_PADHV */
+#define OPpTRUEBOOL		32	/* %hash in (%hash || $foo) in
+					   void context */
+#define OPpMAYBE_TRUEBOOL	64	/* %hash in (%hash || $foo) where
+					   cx is not known till run time */
 
   /* OP_SUBSTR only */
 #define OPpSUBSTR_REPL_FIRST	16	/* 1st arg is replacement string */
@@ -748,12 +753,8 @@ struct opslab {
 
 # define OPSLOT_HEADER		STRUCT_OFFSET(OPSLOT, opslot_op)
 # define OPSLOT_HEADER_P	(OPSLOT_HEADER/sizeof(I32 *))
-# ifdef DEBUGGING
-#  define OpSLOT(o)		(assert(o->op_slabbed), \
+# define OpSLOT(o)		(assert_(o->op_slabbed) \
 				 (OPSLOT *)(((char *)o)-OPSLOT_HEADER))
-# else
-#  define OpSLOT(o)		((OPSLOT *)(((char *)o)-OPSLOT_HEADER))
-# endif
 # define OpSLAB(o)		OpSLOT(o)->opslot_slab
 # define OpslabREFCNT_dec(slab)      \
 	(((slab)->opslab_refcnt == 1) \
