@@ -231,7 +231,9 @@ d_inetntop='undef'
 d_inetpton='undef'
 d_int64_t='undef'
 d_ip_mreq='undef'
+d_ip_mreq_source='undef'
 d_ipv6_mreq='undef'
+d_ipv6_mreq_source='undef'
 d_isascii='undef'
 d_isblank='undef'
 d_isfinite='undef'
